<template>
  <div class="relative h-80 w-full">
    <slot v-bind="{ options: chartOptions }" />
    <div
      v-show="tooltipData"
      ref="tooltip"
      :style="floatingStyles"
      class="pointer-events-none absolute z-10 hidden whitespace-nowrap rounded border border-gray-200 bg-white text-sm font-normal drop-shadow-xl sm:inline-block"
    >
      <slot
        name="tooltip"
        v-bind="{ data: tooltipData }"
      >
        <div class="px-4 py-3">
          <div class="mb-2 text-sm font-medium text-gray-900">
            {{ tooltipData?.title }}
          </div>
          <template
            v-for="(item, index) in tooltipData?.values"
            :key="index"
          >
            <div class="flex items-center text-gray-800">
              <div class="grow-0">
                <div
                  :style="{ borderColor: item.color }"
                  class="h-2.5 w-2.5 rounded-full border-2"
                />
              </div>
              <div class="ml-1.5 flex grow items-center">
                <div class="mr-2 grow-0 font-medium">
                  {{ item.value }}
                </div>
                <div class="grow">
                  {{ item.label }}
                </div>
              </div>
            </div>
          </template>
        </div>
      </slot>
    </div>
  </div>
</template>

<script lang="ts" setup>
import useChart, { type TooltipData } from "@/js/contexts/shared/composables/useChart";
import { autoPlacement, offset, shift, useFloating } from "@floating-ui/vue";
import { ArcElement, CategoryScale, Chart as ChartJS, type ChartOptions, Filler, LinearScale, LineElement, PointElement, RadialLinearScale, Tooltip } from "chart.js";

const props = defineProps<{
  options?: ChartOptions;
}>();

ChartJS.register(CategoryScale, LinearScale, LineElement, RadialLinearScale, ArcElement, Tooltip, PointElement, Filler);

const tooltip = ref<HTMLElement>();
const { tooltipData, createTooltip, tooltipPosition } = useChart();
const { floatingStyles } = useFloating(tooltipPosition, tooltip, { placement: "right", middleware: [shift(), offset(8), autoPlacement()] });

defineSlots<{
  default(props: { options: ChartOptions<any> }): void;
  tooltip(props: { data: TooltipData }): void;
  footer(): void;
}>();

const chartOptions = reactive({
  maintainAspectRatio: false,
  animation: false,
  plugins: {
    tooltip: {
      enabled: false,
      external: createTooltip,
    },
  },
  ...props.options,
} satisfies ChartOptions);
</script>
